<?php
/**
 * Plugin Name: metaps PAYMENT for WooCommerce
 * Plugin URI: https://wordpress.org/plugins/woo-paydesign/
 * Description: metaps PAYMENT (before PAYDESIGN) gateway payment for WooCommerce. 
 * Version: 1.1.24
 * Author: Artisan Workshop
 * Author URI: https://wc.artws.info/
 * Requires at least: 5.0.0
 * Tested up to: 5.8.0
 * WC requires at least: 2.6.0
 * WC tested up to: 5.7.0
 *
 * Text Domain: woo-paydesign
 * Domain Path: /i18n/
 *
 * @package woo-paydesign
 * @category Core
 * @author Artisan Workshop
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

if ( ! class_exists( 'WC4JP_PAYDESIGN' ) ) :

class WC4JP_PAYDESIGN{

	/**
	 * metaps PAYMENT version.
	 *
	 * @var string
	 */
	public $version = '1.1.24';
	/**
	 * WooCommerce Constructor.
	 * @access public
	 * @return WooCommerce
	 */
	public function __construct() {
		// Include required files
		$this->includes();
		$this->init();
		// rated appeal
		add_action( 'wp_ajax_wc4jp_paydesign_rated', array( __CLASS__, 'wc4jp_paydesign_rated') );
		add_filter( 'admin_footer_text', array( $this, 'admin_footer_text' ), 1 );
		// WooCommerce for Japan version
		define( 'WC4JP_METAPS_VERSION', $this->version );
        define( 'WC_METAPS_PLUGIN_URL', plugin_dir_url( __FILE__ ) );
	}
	/**
	 * Include required core files used in admin and on the frontend.
	 */
	private function includes() {
		// Config Setting
		// Credit Card
		define( 'PAYDESIGN_CC_SALES_URL', 'https://www.paydesign.jp/settle/settle3/bp3.dll' );
		define( 'PAYDESIGN_CC_SALES_USER_URL', 'https://www.paydesign.jp/settle/settlex/credit2.dll' );
		define( 'PAYDESIGN_CC_SALES_COMP_URL', 'https://www.paydesign.jp/settle/Fixation/crDkakutei.dll' );
		define( 'PAYDESIGN_CC_SALES_CANCEL_URL', 'https://www.paydesign.jp/settle/Fixation/canauthp.dll' );
		define( 'PAYDESIGN_CC_SALES_REFUND_URL', 'https://www.paydesign.jp/settle/Fixation/cantorip.dll' );
		define( 'PAYDESIGN_CC_SALES_AUTH_URL', 'https://www.paydesign.jp/settle/inquiry/reskaricr.dll' );
		define( 'PAYDESIGN_CC_SALES_CHECK_URL', 'https://www.paydesign.jp/settle/inquiry/result3.dll' );
		// Convenience Store
		define( 'PAYDESIGN_CS_SALES_URL', 'https://www.paydesign.jp/settle/settle2/ubp3.dll' );
		define( 'PAYDESIGN_CS_CANCEL_URL', 'https://www.paydesign.jp/settle/Fixation/can_cvs.dll' );


		// metaps PAYMENT Payment Gateway
		if(get_option('wc_paydesign_cc')){
			include_once( 'includes/gateways/paydesign/class-wc-gateway-paydesign-cc.php' );	// Credit Card
			include_once( 'includes/gateways/paydesign/class-wc-gateway-addon-paydesign-cc.php' );	// Credit Card Subscription
		}
		if(get_option('wc_paydesign_cc_token')){
			include_once( 'includes/gateways/paydesign/class-wc-gateway-paydesign-cc-token.php' );// Credit Card with Token
			include_once( 'includes/gateways/paydesign/class-wc-gateway-addon-paydesign-cc-token.php' );// Credit Card with Token Subscription
		}
		if(get_option('wc_paydesign_cs')) include_once( 'includes/gateways/paydesign/class-wc-gateway-paydesign-cs.php' );	// Convenience store
		if(get_option('wc_paydesign_pe')) include_once( 'includes/gateways/paydesign/class-wc-gateway-paydesign-pe.php' );	// Pay-Easy

		// Admin Setting Screen 
		include_once( 'includes/class-wc-admin-screen-paydesign.php' );
	}
	/**
	 * Init WooCommerce when WordPress Initialises.
	 */
	public function init() {
		// Set up localisation
		$this->load_plugin_textdomain();
	}

	/*
	 * Load Localisation files.
	 *
	 * Note: the first-loaded translation file overrides any following ones if the same translation is present
	 */
	public function load_plugin_textdomain() {
		$locale = apply_filters( 'plugin_locale', get_locale(), 'woo-paydesign' );
		// Global + Frontend Locale
		load_plugin_textdomain( 'woo-paydesign', false, plugin_basename( dirname( __FILE__ ) ) . "/i18n" );
	}
	/**
	 * Change the admin footer text on WooCommerce for Japan admin pages.
	 *
	 * @since  1.1
	 * @param  string $footer_text
	 * @return string
	 */
	public function admin_footer_text( $footer_text ) {
		if ( ! current_user_can( 'manage_woocommerce' ) ) {
			return;
		}
		$current_screen = get_current_screen();
		$wc4jp_paydesign_pages = 'wc4jp-paydesign-output';
		// Check to make sure we're on a WooCommerce admin page
		if ( isset( $current_screen->id ) && $current_screen->id == $wc4jp_paydesign_pages ) {
			if ( ! get_option( 'wc4jp_paydesign_admin_footer_text_rated' ) ) {
				$footer_text = sprintf( __( 'If you like <strong>metaps PAYMENT for WooCommerce.</strong> please leave us a %s&#9733;&#9733;&#9733;&#9733;&#9733;%s rating. A huge thanks in advance!', 'woo-paydesign' ), '<a href="https://wordpress.org/support/plugin/woocommerce-for-japan/reviews/#postform" target="_blank" class="wc4jp-rating-link" data-rated="' . esc_attr__( 'Thanks :)', 'woocommerce-for-japan' ) . '">', '</a>' );
				wc_enqueue_js( "
					jQuery( 'a.wc4jp-rating-link' ).click( function() {
						jQuery.post( '" . WC()->ajax_url() . "', { action: 'wc4jp_paydesign_rated' } );
						jQuery( this ).parent().text( jQuery( this ).data( 'rated' ) );
					});
				" );
			}else{
				$footer_text = __( 'Thank you for selling with WooCommerce for metaps PAYMENT.', 'woo-paydesign' );
			}
		}
		return $footer_text;
	}
	/**
	 * Triggered when clicking the rating footer.
	 */
	public static function wc4jp_paydesign_rated() {
		if ( ! current_user_can( 'manage_woocommerce' ) ) {
			die(-1);
		}

		update_option( 'wc4jp_paydesign_admin_footer_text_rated', 1 );
		die();
	}
}

endif;

/**
 * Load plugin functions.
 */
add_action( 'plugins_loaded', 'WC4JP_PAYDESIGN_plugin', 0 );

function WC4JP_PAYDESIGN_plugin() {
	if ( in_array( 'woocommerce/woocommerce.php', get_option( 'active_plugins' ) ) ) {
		$wc4jp_paydesign = new WC4JP_PAYDESIGN();
	} else {
		add_action( 'admin_notices', 'wc4jp_paydesign_fallback_notice' );
	}
}

//If WooCommerce Plugins is not activate notice
function wc4jp_paydesign_fallback_notice(){
	?>
	<div class="error">
		<ul>
			<li><?php echo __( 'WooCommerce for metaps PAYMENT Payment method is enabled but not effective. It requires WooCommerce in order to work.', 'woo-paydesign' );?></li>
		</ul>
	</div>
	<?php
}
